export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);
        this.setProperty(Property.Minimum, 0);
        this.setProperty(Property.Maximum, 100);

        this.addLevel(20);
        this.addLevel(80);

        this.addInput("KPeriod", InputType.int, 5);
        this.addInput("DPeriod", InputType.int, 3);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            MainBuffer : this.addBuffer(),
            SignalBuffer : this.addBuffer(),
        };

        this.arr = {
            HighesBuffer : [],
            LowesBuffer : [],
        };

        this.vars = {
            drawBegin : 0
        };
    }

    onInit() {
        let {KPeriod, DPeriod} = this.getInputs(),
            short_name = "FastStoch(" + KPeriod + "," + DPeriod + ")";

        this.buffers.MainBuffer
            .setShape(Shape.Line)
            .setColor(Color.Blue)
            .setShortName(short_name)
            .setDrawBegin(KPeriod)
            .setLabel(short_name)
        ;

        this.buffers.SignalBuffer
            .setShape(Shape.Line)
            .setColor(Color.Red)
            .setShortName(short_name)
            .setDrawBegin(KPeriod + DPeriod)
            .setLabel("Signal")
        ;

        this.vars.drawBegin = KPeriod + DPeriod;
    }

    async onUpdate() {
        const {KPeriod, DPeriod, PriceType} = this.getInputs(),
            {HighesBuffer, LowesBuffer} = this.arr,
            {MainBuffer, SignalBuffer} = this.buffers,
            {drawBegin} = this.vars,
            {High, Low, Close} = await Bar.load([Bar.Mode.High, Bar.Mode.Low, Bar.Mode.Close], PriceType),
            barSize = Close.length;

        let i, k, price,min, max, limit, list,
            countedBars = Bar.counted();

        if (barSize <= drawBegin) {
            return 0;
        }
        if (countedBars < 1) {
            for (i = 1; i <= KPeriod; i++) {
                MainBuffer.set(barSize - i,  0);
            }
            for (i = 1; i <= drawBegin; i++) {
                SignalBuffer.set(barSize - i, 0);
            }
        }
        i = barSize - KPeriod;
        if (countedBars > KPeriod) {
            i = barSize - countedBars - 1;
        }
        while (i >= 0) {
            min = 1000000;
            k = i + KPeriod - 1;
            while (k >= i) {
                price = Low[k];
                if (min > price)
                    min = price;
                k--;
            }
            LowesBuffer[i]= min;
            i--;
        }
        i = barSize - KPeriod;
        if (countedBars > KPeriod) {
            i = barSize - countedBars - 1;
        }
        while (i >= 0) {
            max = -1000000;
            k = i + KPeriod - 1;
            while (k >= i) {
                price = High[k];
                if (max < price){
                    max = price;
                }

                k--;
            }
            HighesBuffer[i] = max;
            i--;
        }
        i = barSize - KPeriod;
        if (countedBars > KPeriod) {
            i = barSize - countedBars - 1;
        }

        while (i >= 0) {
            if ((HighesBuffer[i] - LowesBuffer[i]) !== 0) {
                MainBuffer.set(i, (100 * (Close[i] - LowesBuffer[i]) / (HighesBuffer[i] - LowesBuffer[i])));
            }
            i--;
        }
        if (countedBars > 0) {
            countedBars--;
        }

        limit = barSize - countedBars;
        list = Indicators.MAOnArray(MainBuffer, barSize, DPeriod, 0, Averaging.Simple);
        SignalBuffer.fill(list);
    }
}
